from NLP.Tokenizer.Tokenizer import Tokenize


def test_tokenizer():
    tokenizer = Tokenize()
    assert ["این", "یک", "تست", "است"] == tokenizer.space_tokenize("این یک تست است")


def test_stanfordTokenize():
    tokenizer = Tokenize(stanford_tokenize=True)
    assert ['این', 'یک', 'تست', 'است'] == tokenizer.stanford_tokenize("این یک تست است")


def test_ngramTokenize():
    tokenizer = Tokenize()
    assert ['این یک', 'یک تست', 'تست است'] == tokenizer.ngramTokenize("این یک تست است", 2)


def test_wordseg():
    tokenizer = Tokenize(word_breaker=True)
    assert ["این", "یک", "تست", "است"] == tokenizer.word_segmentation("اینیکتستاست")


def test_wordcomponent():
    tokenizer = Tokenize(component_words=True)
    assert ["دانش‌آموز"] == tokenizer.component_words("دانش آموز")



"""
‫با توجه به استفاده این فایل دو تا تابع داره که یکی برای نرمال کردن پرس‌وجو است و دیگری برای نرمال کردن متن
"""
import re
import pandas as pd
import pathlib

# جداولی که آقای جمال در اختیارمون گذاشتن
charTable = pd.read_csv('{}/'.format(pathlib.Path(__file__).parent.absolute()) + 'tables/chars.csv', delimiter=',')
SW = pd.read_csv('{}/'.format(pathlib.Path(__file__).parent.absolute()) + 'tables/specialWords.csv', delimiter=',')

maketrans = lambda A, B: dict((ord(a), b) for a, b in zip(A, B))
compile_patterns = lambda patterns: [(re.compile(pattern), repl) for pattern, repl in patterns]


# ‫این کلاس تمامی به هنجارسازی های گفته شده در readme را انجام می‌دهد
class Normalizer(object):
    def __init__(self, remove_extra_spaces=True, persian_numbers=True, unicode=True, word_refine=True,
                 persian_style=True, punctuation_spacing=True, affix_spacing=True, remove_emoji=True,
                 remove_diacritics=True, remove_extra_char=True, remove_punctuation=False, remove_repeated_word=False):
        self._punctuation_spacing = punctuation_spacing
        self._affix_spacing = affix_spacing
        self._remove_emoji = remove_emoji
        self._remove_extra_char = remove_extra_char
        # ‫تبدیل اعدادفارسی  به انگلیسی
        translation_src, translation_dst = ' ىكي“”', ' یکی""'
        if persian_numbers:
            translation_src += '۰۱۲۳۴۵۶۷۸۹٪'
            translation_dst += '0123456789%'
        if unicode:
            unicode_chars = ''.join(list(charTable['unicode']))
            translation_src += unicode_chars
            original_chars = ''.join(list(charTable['orginal']))
            translation_dst += original_chars
        self.translations = maketrans(translation_src, translation_dst)

        self.character_refinement_patterns = []

        if remove_diacritics:
            self.character_refinement_patterns.append(
                ('[\u064B\u064C\u064D\u064E\u064F\u0650\u0651\u0652]', ''),
                # remove FATHATAN, DAMMATAN, KASRATAN, FATHA, DAMMA, KASRA, SHADDA, SUKUN
            )

        if word_refine:
            self.character_refinement_patterns.extend(list(zip(SW['before'], SW['after'])))
        if persian_style:
            self.character_refinement_patterns.extend([
                ('"([^\n"]+)"', r'«\1»'),  # replace quotation with gyoome
                ('([\d+])\.([\d+])', r'\1٫\2'),  # replace dot with momayez
                (r' ?\.\.\.', ' …'),  # replace 3 dots
            ])
        if remove_extra_spaces:
            self.character_refinement_patterns.extend([
                (r' +', ' '),  # remove extra spaces
                (r'\n\n+', '\n'),  # remove extra newlines
                (r'\r\n+', '\r\n'),  # remove extra newlines
                (r'[ـ\r]', ''),  # remove keshide, carriage returns
                (r'\s+\n', '\n'),  # end of the paragraph delete space before \n
                (r'(\w)(\1{2,})', r'\1'),  # remove repeating of a chars
            ])
        if remove_punctuation:
            self.character_refinement_patterns.extend([
                (r'[^\w\s]', '')  # remove punctuations
            ])
        if remove_repeated_word:
            self.character_refinement_patterns.extend([
                (r'\b(\w+)( \1\b)+', r'\1')
            ])
        self.character_refinement_patterns = compile_patterns(self.character_refinement_patterns)

        punc_after, punc_before = r'\.:!،abcdefghijklmnopqrstuvwxyz؛؟»\]\)\}0123456789', r'«\[\(\{abcdefghijklmnopqrstuvwxyz0123456789'
        if punctuation_spacing:
            self.punctuation_spacing_patterns = compile_patterns([
                ('" ([^\n"]+) "', r'"\1"'),  # remove space before and after quotation
                (' ([' + punc_after + '])', r'\1'),  # remove space before
                ('([' + punc_before + ']) ', r'\1'),  # remove space after
                ('([' + punc_after[:3] + '])([^ ' + punc_after + '\d0123456789])', r'\1 \2'),
                # put space after . and :
                ('([' + punc_after[3:] + '])([^ ' + punc_after + '])', r'\1 \2'),  # put space after
                ('([^ ' + punc_before + '])([' + punc_before + '])', r'\1 \2'),  # put space before
            ])

        if affix_spacing:
            self.affix_spacing_patterns = compile_patterns([
                (r'([^ ]ه) ی ', r'\1‌ی '),  # fix ی space
                (r'(^| )(ن?می) ', r'\1\2'),  # put zwnj after می, نمی
                (
                    r'(?<=[^\n\d ' + punc_after + punc_before + ']{2}) (تر(ین?)?|گری?|های?)(?=[ \n' + punc_after + punc_before + ']|$)',
                    r'\1'),  # put zwnj before تر, تری, ترین, گر, گری, ها, های
                (r'([^ ]ه) (ا(م|یم|ش|ند|ی|ید|ت))(?=[ \n' + punc_after + ']|$)', r'\1‌\2'),
                # join ام, ایم, اش, اند, ای, اید, ات

            ])

    def character_refinement(self, text):
        text = text.translate(self.translations)
        for pattern, repl in self.character_refinement_patterns:
            text = pattern.sub(repl, text)
        return text

    # ‫حذف اموجی‌ها
    def deEmojify(self, text):
        regrex_pattern = re.compile(pattern="["
                                            u"\U0001F600-\U0001F64F"  # emoticons
                                            u"\U0001F300-\U0001F5FF"  # symbols & pictographs
                                            u"\U0001F680-\U0001F6FF"  # transport & map symbols
                                            u"\U0001F1E0-\U0001F1FF"  # flags (iOS)
                                            u"\U00002500-\U00002BEF"  # chinese char
                                            u"\U00002702-\U000027B0"
                                            u"\U00002702-\U000027B0"
                                            u"\U000024C2-\U0001F251"
                                            u"\U0001f926-\U0001f937"
                                            u"\U00010000-\U0010ffff"
                                            u"\u2640-\u2642"
                                            u"\u2600-\u2B55"
                                            u"\u200d"
                                            u"\u23cf"
                                            u"\u23e9"
                                            u"\u231a"
                                            u"\ufe0f"  # dingbats
                                            u"\u3030"
                                            "]+", flags=re.UNICODE)
        return regrex_pattern.sub(r' ', text)

    # ‫این تابع برای قرار دادن space بعد از punctuation هاست
    def punctuation_spacing(self, text):
        for pattern, repl in self.punctuation_spacing_patterns:
            text = pattern.sub(repl, text)
        return text

    # این تابع برای اصلاح نیم‌فاصله‌هاست
    def affix_spacing(self, text):
        for pattern, repl in self.affix_spacing_patterns:
            text = pattern.sub(repl, text)
        return text

    # تابع اصلی برای نرمال کردن متن
    def normalizeText(self, text):
        if self._remove_emoji:
            text = self.deEmojify(text).strip()
        text = self.character_refinement(text)
        if self._punctuation_spacing:
            text = self.punctuation_spacing(text)
        if self._affix_spacing:
            text = self.affix_spacing(text)
        return text


import stanza
from symspellpy.symspellpy import SymSpell
import pickle
import re
import pathlib
import os
# ‫استفاده از یکسان‌سازی که خودمان توسعه دادیم
from ..Normalizer.Normalizer import Normalizer

with open('{}/'.format(pathlib.Path(__file__).parent.absolute()) + 'LM/dic_uni.pickle', 'rb') as f_uni:
    unigram = pickle.load(f_uni)

with open('{}/'.format(pathlib.Path(__file__).parent.absolute()) + 'LM/dic_bi.pickle', 'rb') as f_bi:
    bigram = pickle.load(f_bi)


# ‫این کلاس  شامل دو مدل واحدساز است یکی استفاده از واحدساز stanford  و ngram tokenizer
class Tokenize(object):

    def __init__(self, word_breaker=False, stanford_tokenize=False, component_words=False):
        if word_breaker:
            self.sym_spell = SymSpell(max_dictionary_edit_distance=0, prefix_length=7)
            self.dictionary_path = '{}/'.format(pathlib.Path(__file__).parent.absolute()) + \
                                   "frequency_bigramdictionary2_fa_243_342.txt"
            self.sym_spell.load_dictionary(self.dictionary_path, term_index=0, count_index=2)
        if stanford_tokenize:
            if not os.path.exists('/home/dev/stanza_resources/fa'):
                print('downloading pipline ...')
                stanza.download('fa')
            self.nlp = stanza.Pipeline(lang='fa', processors='tokenize,mwt')
        self.tokens = []
        self.my_normalizer = Normalizer()
        self.uni_prop = {}
        self.bi_prop = {}

        # it takes a long time
        if component_words:
            for key, value in unigram.items():
                self.uni_prop[key] = value / sum(unigram.values())

            for key, value in bigram.items():
                condition = key.split()[0]
                self.bi_prop[key] = (value / unigram[condition]) * self.uni_prop[condition]

    # ‫این تابع برای استفاده از واحدساز stanfordnlp است
    def stanford_tokenize(self, text):

        normalized_text = self.my_normalizer.normalizeText(text)
        doc = self.nlp(normalized_text)
        for sent in doc.sentences:
            for word in sent.words:
                self.tokens.append(word.text)
        return self.tokens

    def space_tokenize(self, text):
        normalized_text = self.my_normalizer.normalizeText(text)
        self.tokens = normalized_text.split(" ")
        return self.tokens

    # ‫این تابع برای ngram tokenizer است که n را به صورت ورودی دریافت می‌کند
    def ngramTokenize(self, text, n):
        normalized_text = self.my_normalizer.normalizeText(text)
        words = normalized_text.split()
        ngrams = zip(*[words[i:] for i in range(n)])
        self.tokens = [" ".join(ngram) for ngram in ngrams]
        return self.tokens

    def word_segmentation(self, text):
        result = self.sym_spell.word_segmentation(text)
        return self.space_tokenize(result.corrected_string)

    @staticmethod
    def ngram(text, n):
        words = text.split()
        ngrams = zip(*[words[i:] for i in range(n)])
        tokens = [" ".join(ngram) for ngram in ngrams]
        return tokens

    @staticmethod
    def perplexity(prob, n):
        perplexity = pow(1 / prob, 1 / n)
        return perplexity

    # ‫استفاده از مدلهای زبانی برای تشخیص قراردادن نیم فاصله
    def component_words(self, text):
        unigram_sample = self.ngram(text, 1)
        bigram_sample = self.ngram(text, 2)

        lst_sample_uni = [self.uni_prop[word] for word in unigram_sample if word in self.uni_prop.keys()]
        lst_sample_bi = [self.bi_prop[bi] for bi in bigram_sample if bi in self.bi_prop.keys()]

        dic_uni_per = {}
        dic_bi_per = {}

        for i, word in enumerate(unigram_sample):
            try:
                dic_uni_per[word] = self.perplexity(lst_sample_uni[i], 1)
            except:
                continue

        for i, bi in enumerate(bigram_sample):
            try:
                dic_bi_per[bi] = self.perplexity(lst_sample_bi[i], 2)
            except:
                continue

        token = []
        num_of_append = 0
        if len(unigram_sample) >= 2:
            for i in range(len(unigram_sample)):
                word1 = unigram_sample[i]
                try:
                    word2 = unigram_sample[i + 1]
                except:
                    if num_of_append == len(unigram_sample) + 1:
                        break
                    else:
                        token.append(word1)
                        break
                bi = " ".join([word1, word2])
                if bi in dic_bi_per.keys() and dic_bi_per[bi] < dic_uni_per[word1] and dic_bi_per[bi] < dic_uni_per[
                    word2]:
                    bi = "‌".join([word1, word2])
                    token.append(bi)
                    num_of_append = num_of_append + 2
                else:
                    token.append(word1)
                    num_of_append = num_of_append + 1
        else:
            token = unigram_sample
        s = " ".join(token)
        s = re.sub(r'\b(\w+)( \1\b)+', r'\1', s)
        return self.space_tokenize(s)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="similarity_model_trainer_2021", # Replace with your own username
    version="0.1.3",
    author="Author",
    author_email="",
    description="A small example package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="",
    packages=setuptools.find_packages(),
    include_package_data=True,
    package_data={'data': ['similarity_model_trainer_2021/Normailzer/tables/chars.csv',
                           'similarity_model_trainer_2021/Normailzer/tables/specialWords.csv',
                           'similarity_model_trainer_2021/Tokenizer/frequency_bigramdictionary2_fa_243_342.txt',
                           'similarity_model_trainer_2021/Tokenizer/LM/dic_bi.pickle',
                           'similarity_model_trainer_2021/Tokenizer/LM/dic_uni.pickle']},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


from NLP.Normalizer import Normalizer
normalizer = Normalizer()


def test_1():
    assert "من به بازار میروم" == normalizer.normalizeText("من به بازار می   روم")


def test_2():
    assert "مهدی بابایی کلا کارهای سخت دوست دارد" == normalizer.normalizeText(
        "مهدیِ بابایی کُلاً کار های سخت دوست دارد")


def test_3():
    assert "شدت عملش زیاد بود" == normalizer.normalizeText("شدّت عملش زیاد بود")



def test_5():
    assert "او گفت «سلام» و رفت" == normalizer.normalizeText("او گفت “ سلام ” و رفت")


def test_6():
    assert "کتاب هنر جنگ بسیار عالی است" == normalizer.normalizeText("کتاب هنر جنگ بسیار عاااااالی است")


def test_7():
    assert "این یک تست است\nاین یک تست است\nاین یک تست است" == normalizer.normalizeText("این یک تست است\n\nاین یک تست "
                                                                                        "است\n\nاین یک تست است")

def test_8():
    assert "واقعا مطلب سنگینی بود من خیلی لذت بردم" == normalizer.normalizeText(
        "واقعا مطلب سنگینی بود😂😂😂😂من خیلی لذت بردم")


def test_9():
    assert "امروز استقلال برنده است" == normalizer.normalizeText(
        "💙💙🌟🌟💙💙امروز استقلال برنده است 🧿🧿🧿🧿🧿🧿🧿🧿🧿🧿🧿🧿🧿")


def test_10():
    assert "فاتح اعداد لنگ شاه شاهان عدد الانم همیشه باعث سوزشی مرسی که هستی" == normalizer.normalizeText(
        "فاتح اعداد لنگ شاه شاهان 😍😍😍💙💙💙💙 عدد ✌✌ الانم ✌✌✌ همیشه باعث سوزشی مرسی که هستی😍😍💙💙💙")
